//////////////////////////////////////////////////////////////////////////////
//
//    Project : BRAM_LED
// Description: This design outputs LED patterns based on two reconfigurable
//              modules.  The recon_block_bram module ouput depends on the
//              BRAM contents within the reconfig module.  The recon_block_counter
//              module output depends on the counter within the reconfig
//              module.  Partial bit files can be created and downloaded that
//              change the contents of the BRAM and the behavior of the counter.
//              The counter reconfig module containes I/O that are embedded
//              within the reconfig module.
//
//////////////////////////////////////////////////////////////////////////////

// black box definition for reconfigurable module recon_block_bram

module recon_block_bram(
    en, clk, addr,
    data_out);

    input  en; 
    input  clk;   
    input  [11:0] addr;     
    output [7:0]  data_out;   
endmodule


// black box definition for reconfigurable module recon_block_counter

module recon_block_count(
    rst_n, clk,
    led_out);

    input  rst_n; 
    input  clk;   
    output [3:0] led_out;   
endmodule

//////////////////////////////////////////////////////////////////////////////
//  Top-level, static design
//////////////////////////////////////////////////////////////////////////////

module top(
    rst_n, clk_n, clk_p,
    test_count,
    out_counter,
    out_bram);
    
    input rst_n;               // Reset mapped to center push button
    input clk_n;               // 200MHz input clock
    input clk_p;               // 200MHz input clock
    output reg test_count;     // LED to see clock, counter and reset activity
    output [7:0] out_bram;     // mapped to general purpose LEDs

    // turn off IOBUF insertion for embedded IO with XST syntax
    // synthesis attribute buffer_type out_counter none;
    output [3:0] out_counter;  // mapped to push button LEDs

    reg [34:0]  count;

    // instantiate static modules and global logic

    IBUFGDS clkin1_buf(
    	.O  (clkin1),
   	.I  (clk_p),
    	.IB (clk_n));

    wire        locked_unused;
    wire        clkfbout;
    wire        clkfbout_buf;
    wire        clkfboutb_unused;
    wire        clkout0;
    wire        clkout0b_unused;
    wire        clkout1_unused;
    wire        clkout1b_unused;
    wire        clkout2_unused;
    wire        clkout2b_unused;
    wire        clkout3_unused;
    wire        clkout3b_unused;
    wire        clkout4_unused;
    wire        clkout5_unused;
    wire        clkout6_unused;


    MMCM_BASE #(
      .BANDWIDTH("OPTIMIZED"),   // Jitter programming ("HIGH","LOW","OPTIMIZED")
      .CLKFBOUT_MULT_F(6.0),     // Multiply value for all CLKOUT (5.0-64.0).
      .CLKFBOUT_PHASE(0.0),      // Phase offset in degrees of CLKFB (0.00-360.00).
      .CLKIN1_PERIOD(0.0),       // Input clock period in ns to ps resolution (i.e. 33.333 is 30 MHz).
      .CLKOUT0_DIVIDE_F(12.0),    // Divide amount for CLKOUT0 (1.000-128.000).
      // CLKOUT0_DUTY_CYCLE - CLKOUT6_DUTY_CYCLE: Duty cycle for each CLKOUT (0.01-0.99).
      .CLKOUT0_DUTY_CYCLE(0.5),
      .CLKOUT1_DUTY_CYCLE(0.5),
      .CLKOUT2_DUTY_CYCLE(0.5),
      .CLKOUT3_DUTY_CYCLE(0.5),
      .CLKOUT4_DUTY_CYCLE(0.5),
      .CLKOUT5_DUTY_CYCLE(0.5),
      .CLKOUT6_DUTY_CYCLE(0.5),
      // CLKOUT0_PHASE - CLKOUT6_PHASE: Phase offset for each CLKOUT (-360.000-360.000).
      .CLKOUT0_PHASE(0.0),
      .CLKOUT1_PHASE(0.0),
      .CLKOUT2_PHASE(0.0),
      .CLKOUT3_PHASE(0.0),
      .CLKOUT4_PHASE(0.0),
      .CLKOUT5_PHASE(0.0),
      .CLKOUT6_PHASE(0.0),
      // CLKOUT1_DIVIDE - CLKOUT6_DIVIDE: Divide amount for each CLKOUT (1-128)
      .CLKOUT1_DIVIDE(1),
      .CLKOUT2_DIVIDE(1),
      .CLKOUT3_DIVIDE(1),
      .CLKOUT4_DIVIDE(1),
      .CLKOUT5_DIVIDE(1),
      .CLKOUT6_DIVIDE(1),
      .CLKOUT4_CASCADE("FALSE"), // Cascase CLKOUT4 counter with CLKOUT6 (TRUE/FALSE)
      .CLOCK_HOLD("FALSE"),      // Hold VCO Frequency (TRUE/FALSE)
      .DIVCLK_DIVIDE(1),         // Master division value (1-80)
      .REF_JITTER1(0.0),         // Reference input jitter in UI (0.000-0.999).
      .STARTUP_WAIT("FALSE")     // Not supported. Must be set to FALSE.
   )
   MMCM_BASE_inst (
      // Clock Outputs: 1-bit (each) output: User configurable clock outputs
      .CLKOUT0(clkout0),     // 1-bit output: CLKOUT0 output
      .CLKOUT0B(clkout0b_unused),   // 1-bit output: Inverted CLKOUT0 output
      .CLKOUT1(clkout1_unused),     // 1-bit output: CLKOUT1 output
      .CLKOUT1B(clkout1b_unused),   // 1-bit output: Inverted CLKOUT1 output
      .CLKOUT2(clkout2_unused),     // 1-bit output: CLKOUT2 output
      .CLKOUT2B(clkout2b_unused),   // 1-bit output: Inverted CLKOUT2 output
      .CLKOUT3(clkout3_unused),     // 1-bit output: CLKOUT3 output
      .CLKOUT3B(clkout3b_unused),   // 1-bit output: Inverted CLKOUT3 output
      .CLKOUT4(clkout4_unused),     // 1-bit output: CLKOUT4 output
      .CLKOUT5(clkout5_unused),     // 1-bit output: CLKOUT5 output
      .CLKOUT6(clkout6_unused),     // 1-bit output: CLKOUT6 output
      // Feedback Clocks: 1-bit (each) output: Clock feedback ports
      .CLKFBOUT(clkfbout),   // 1-bit output: Feedback clock output
      .CLKFBOUTB(clkfboutb_unused), // 1-bit output: Inverted CLKFBOUT output
      // Status Port: 1-bit (each) output: MMCM status ports
      .LOCKED(locked_unused),       // 1-bit output: LOCK output
      // Clock Input: 1-bit (each) input: Clock input
      .CLKIN1(clkin1),
      // Control Ports: 1-bit (each) input: MMCM control ports
      .PWRDWN(1'b0),       // 1-bit input: Power-down input
      .RST(1'b0),             // 1-bit input: Reset input
      // Feedback Clocks: 1-bit (each) input: Clock feedback ports
      .CLKFBIN(clkfbout_buf)      // 1-bit input: Feedback clock input
   );

  BUFG clkf_buf (
    .O (clkfbout_buf),
    .I (clkfbout));


  BUFG clkout1_buf (
    .O (gclk),
    .I (clkout0));


    // instantiate reconfigurable module BRAM
    recon_block_bram U1_RP_Bram (
      .en       (rst_n),
      .clk      (gclk),
      .addr     (count[34:23]),
      .data_out (out_bram));

    // instantiate reconfigurable module counter
    recon_block_count U2_RP_Count (
      .rst_n    (rst_n),
      .clk      (gclk),
      .led_out (out_counter));

// MAIN

   always @(posedge gclk)
     if (rst_n)
       begin
         count <= 0;
         test_count <= 0;
       end
     else
       begin
         count <= count + 1;
         test_count <= count[24];
       end

endmodule
