-- #####################################################
-- #
-- #  Partition-based Partial Reconfiguration Reference Design
-- #
-- #  Parimal Patel, Xilinx University Program, San Jose
-- #  October 6, 2011
-- #
-- #  Targeted for ML605 Evaluation Board
-- #
-- #####################################################


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
Library UNISIM;
use UNISIM.vcomponents.all;

-- design for ML605 board talking to a UART 
entity top is
    Port ( clk_p   	: in std_logic;		-- 200MHz system clock
			  clk_n		: in std_logic;
           reset     : in std_logic;
           rx		   : in std_logic;		-- serial data from UART
           tx   		: out std_logic;		-- serial data to UART
           button_start : in std_logic;
           button_stop : in std_logic;
           led       : out std_logic_vector(3 downto 0)
         );	
end top;

architecture Behavioral of top is

component BUFGP
  port ( I : in std_logic;
         O : out std_logic
       ); 
end component;

-- This is a static module of control logic.  It parses keyboard input and
-- sends it out to the rx/tx.  There will be a single instance of it
component control is
    Port (               clk : in std_logic;
                  data_to_op : out std_logic_vector(3 downto 0);
           instruction_to_op : out std_logic_vector(1 downto 0);
                data_from_op : in signed(7 downto 0);
                       op_id : in std_logic;
                    write_tx : out std_logic;
                      din_tx : out std_logic_vector(7 downto 0);
             data_present_rx : in std_logic;
                     read_rx : out std_logic;
                     dout_rx : in std_logic_vector(7 downto 0)
          );
end component;

-- A reconfigurable module area.  The control module speaks to it on data_in and gets 
--     answers back on data_out.  Id is  used to signal  the control module which flavor
--	    of the reconfig module is loaded and running
component rModule_addsub  is
    Port (            clk : in std_logic;
                  data_in : in std_logic_vector(3 downto 0);
           instruction_in : in std_logic_vector(1 downto 0);
                 data_out : out signed(7 downto 0);
                       id : out std_logic
                    );
end component;

-- Static Module to turn chip clock into a baud rate clock to be
--    used by uarts
component uart_baudClock is
    Port (        in_clk : in std_logic;
                baud_clk : out std_logic
         );
end component;

-- Static Module to transmit data off board
component uart_tx is
  port (            din : in STD_LOGIC_VECTOR (7 downto 0);
                  write : in  STD_LOGIC;
           reset_buffer : in  STD_LOGIC;
           en_16_x_baud : in  STD_LOGIC;
                    clk : in  STD_LOGIC;
             serial_out : out STD_LOGIC;
            buffer_full : out STD_LOGIC
           );
end component;

-- Static Module to receive data from off board
component uart_rx is
  port (      serial_in : in  STD_LOGIC;
                   read : in  STD_LOGIC;
           reset_buffer : in  STD_LOGIC;
           en_16_x_baud : in  STD_LOGIC;
                    clk : in  STD_LOGIC;
                   dout : out STD_LOGIC_VECTOR (7 downto 0);
           data_present : out STD_LOGIC;
            buffer_full : out STD_LOGIC
           );
end component;

-- reconfigurable module.  Controls LEDs
component rModule_leds is
   Port	(
      clk : in std_logic;
      reset : in std_logic;
      en : in std_logic;
      led : out std_logic_vector(3 downto 0)
   );
end component;


component led_control is
   Port	(
      clk : in std_logic;
      reset : in std_logic;
      button_start : in std_logic;
      button_stop : in std_logic;
      rst : out std_logic;
      en : out std_logic
   );
end component;

-- signals used in context logic
signal                  clk_int : std_logic;
signal               data_to_op : std_logic_vector(3 downto 0);
signal             data_from_op : signed(7 downto 0);
signal          data_to_control : signed(7 downto 0);
signal        data_from_control : std_logic_vector(3 downto 0);
signal instruction_from_control : std_logic_vector(1 downto 0);
signal        instruction_to_op : std_logic_vector(1 downto 0);
signal                    op_id : std_logic;
signal                       id : std_logic;

-- data to (from) the send (receive) UART
signal          din_tx, dout_rx : std_logic_vector(7 downto 0);

-- tell UART to write or read
signal                 write_tx : std_logic;
signal                  read_rx : std_logic;

-- baud rate clock for transmit and receive UARTs
signal                 baud_clk : std_logic;

-- data avail from receive UART and the  last 2 values
signal          data_present_rx : std_logic;

-- led signals
signal rst : std_logic;
signal en : std_logic;
signal led_omod : std_logic_vector(3 downto 0);

-- clocking related signal
signal clkin1 : std_logic;
signal clkout0 : std_logic;
signal clkfbout : std_logic;
signal clkfbout_buf : std_logic;

begin
-- global clock buffer for the design

   IBUFGDS_inst : IBUFGDS
   generic map (
      IBUF_LOW_PWR => TRUE, -- Low power (TRUE) vs. performance (FALSE) setting for refernced I/O standards
      IOSTANDARD => "DEFAULT")
   port map (
      O => clkin1,  -- Clock buffer output
      I => clk_p,  -- Diff_p clock buffer input
      IB => clk_n -- Diff_n clock buffer input
   );

   MMCM_BASE_inst : MMCM_BASE
   generic map (
      BANDWIDTH => "OPTIMIZED",  -- Jitter programming ("HIGH","LOW","OPTIMIZED")
      CLKFBOUT_MULT_F => 6.0,    -- Multiply value for all CLKOUT (5.0-64.0).
      CLKFBOUT_PHASE => 0.0,     -- Phase offset in degrees of CLKFB (0.00-360.00).
      CLKIN1_PERIOD => 0.0,      -- Input clock period in ns to ps resolution (i.e. 33.333 is 30 MHz).
      CLKOUT0_DIVIDE_F => 12.0,   -- Divide amount for CLKOUT0 (1.000-128.000). 
      -- CLKOUT0_DUTY_CYCLE - CLKOUT6_DUTY_CYCLE: Duty cycle for each CLKOUT (0.01-0.99).
      CLKOUT0_DUTY_CYCLE => 0.5,
      CLKOUT1_DUTY_CYCLE => 0.5,
      CLKOUT2_DUTY_CYCLE => 0.5,
      CLKOUT3_DUTY_CYCLE => 0.5,
      CLKOUT4_DUTY_CYCLE => 0.5,
      CLKOUT5_DUTY_CYCLE => 0.5,
      CLKOUT6_DUTY_CYCLE => 0.5,
      -- CLKOUT0_PHASE - CLKOUT6_PHASE: Phase offset for each CLKOUT (-360.000-360.000).
      CLKOUT0_PHASE => 0.0,
      CLKOUT1_PHASE => 0.0,
      CLKOUT2_PHASE => 0.0,
      CLKOUT3_PHASE => 0.0,
      CLKOUT4_PHASE => 0.0,
      CLKOUT5_PHASE => 0.0,
      CLKOUT6_PHASE => 0.0,
      -- CLKOUT1_DIVIDE - CLKOUT6_DIVIDE: Divide amount for each CLKOUT (1-128)
      CLKOUT1_DIVIDE => 1,
      CLKOUT2_DIVIDE => 1,
      CLKOUT3_DIVIDE => 1,
      CLKOUT4_DIVIDE => 1,
      CLKOUT5_DIVIDE => 1,
      CLKOUT6_DIVIDE => 1,
      CLKOUT4_CASCADE => FALSE,  -- Cascase CLKOUT4 counter with CLKOUT6 (TRUE/FALSE)
      CLOCK_HOLD => FALSE,       -- Hold VCO Frequency (TRUE/FALSE)
      DIVCLK_DIVIDE => 1,        -- Master division value (1-80)
      REF_JITTER1 => 0.0,        -- Reference input jitter in UI (0.000-0.999).
      STARTUP_WAIT => FALSE      -- Not supported. Must be set to FALSE.
   )
   port map (
      -- Clock Outputs: 1-bit (each) output: User configurable clock outputs
      CLKOUT0 => clkout0,     -- 1-bit output: CLKOUT0 output
      CLKOUT0B => open,   -- 1-bit output: Inverted CLKOUT0 output
      CLKOUT1 => open,     -- 1-bit output: CLKOUT1 output
      CLKOUT1B => open,   -- 1-bit output: Inverted CLKOUT1 output
      CLKOUT2 => open,     -- 1-bit output: CLKOUT2 output
      CLKOUT2B => open,   -- 1-bit output: Inverted CLKOUT2 output
      CLKOUT3 => open,     -- 1-bit output: CLKOUT3 output
      CLKOUT3B => open,   -- 1-bit output: Inverted CLKOUT3 output
      CLKOUT4 => open,     -- 1-bit output: CLKOUT4 output
      CLKOUT5 => open,     -- 1-bit output: CLKOUT5 output
      CLKOUT6 => open,     -- 1-bit output: CLKOUT6 output
      -- Feedback Clocks: 1-bit (each) output: Clock feedback ports
      CLKFBOUT => clkfbout,   -- 1-bit output: Feedback clock output
      CLKFBOUTB => open, -- 1-bit output: Inverted CLKFBOUT output
      -- Status Port: 1-bit (each) output: MMCM status ports
      LOCKED => open,       -- 1-bit output: LOCK output
      -- Clock Input: 1-bit (each) input: Clock input
      CLKIN1 => clkin1,
      -- Control Ports: 1-bit (each) input: MMCM control ports
      PWRDWN => '0',       -- 1-bit input: Power-down input
      RST => '0',             -- 1-bit input: Reset input
      -- Feedback Clocks: 1-bit (each) input: Clock feedback ports
      CLKFBIN => clkfbout_buf      -- 1-bit input: Feedback clock input
   );

   BUFG_inst_fb : BUFG
   port map (
      O => clkfbout_buf, -- 1-bit output: Clock buffer output
      I => clkfbout  -- 1-bit input: Clock buffer input
   );

   BUFG_inst_out : BUFG
   port map (
      O => clk_int, -- 1-bit output: Clock buffer output
      I => clkout0  -- 1-bit input: Clock buffer input
   );

-- fixed control module.  This is the logic to receive data from off chip, process it
--	possibly using current flavor of  rModule in place and send data off chip
control_inst : control
	port map (  clk => clk_int,
               data_to_op => data_to_op,
               instruction_to_op => instruction_to_op,
               data_from_op => data_from_op,
               op_id => op_id,
               write_tx => write_tx,
               din_tx => din_tx,
               data_present_rx => data_present_rx,
               read_rx => read_rx,
               dout_rx => dout_rx
              );


-- reconfigurable module.  This can be loaded dynamically with any flavor as long
--	as it speaks to this interface
reconfig_addsub : rModule_addsub  
    port map ( clk => clk_int,
               data_in => data_to_op,
					instruction_in => instruction_to_op,
               data_out => data_from_op,
               id => op_id
              );
				 
-- UART used to transmit data to off chip.  It has three modules.
--     uart_baudClock_inst : convert chip clock to uart 9600 baud rate
--     uart_tx_inst : transmit data out UART port
--     uart_rx_inst : receive data from UART port
uart_baudClock_inst : uart_baudClock
  port map (   in_clk => clk_int,
               baud_clk => baud_clk
           );

uart_tx_inst : uart_tx 
  port map (   din => din_tx,
               write => write_tx,
					reset_buffer => '0',
					en_16_x_baud => baud_clk,
               clk => clk_int,
					serial_out => tx,
					buffer_full => open
           );


uart_rx_inst : uart_rx 
  port map (   serial_in => rx,
               read => read_rx,
					reset_buffer => '0',
					en_16_x_baud => baud_clk,
               clk => clk_int,
               dout => dout_rx,
					data_present => data_present_rx,
					buffer_full => open
           );

led_control_inst : led_control
   Port	map (
					clk => clk_int,
					reset => reset,
					button_start => button_start,
					button_stop => button_stop,
					rst => rst,
					en => en
				);

reconfig_leds : rModule_leds
   Port	map (
					clk => clk_int,
					reset => rst,
					en => en,
					led => led
				);

end Behavioral;
